#!/usr/bin/php
<?php
/*
* Creats a csv report of times spend on issues for
* a specific month of a specific user.
*/

// load the config, create the connection and load the repo sepcified on the cmd line args
// we will then have $client and $repo available
require 'setup.php';

// set the repo username and month to get the report for
echo "Enter username for report: ";
$name = trim(fgets(STDIN)); // read from stdin
$date = date('Y-m', strtotime('now -1 month')); //last month
echo "Generateing report for {$repo->user}/{$repo->repo} of time spent in month $date by $name...\n";
// Build a crazy id to number lookup table
// https://github.com/go-gitea/gitea/issues/8513
foreach ($repo->getIssues() as $issue) {
    $issueNumbers[$issue->id] = $issue->number;
}
foreach ($repo->getPulls() as $pull) {
    $issue = $repo->getIssue($pull->number);
    $issueNumbers[$issue->id] = $issue->number;
}
foreach ($repo->getIssues(['state' => 'closed']) as $issue) {
    $issueNumbers[$issue->id] = $issue->number;
}
foreach ($repo->getPulls(['state' => 'closed']) as $pull) {
    $issue = $repo->getIssue($pull->number);
    $issueNumbers[$issue->id] = $issue->number;
}

// process and sum up all the times by issue number
// (not ID as retrned by API)
$totals = [];
foreach ($repo->getTimesByUsername($name) as $time) {
    // check the month
    if ($date == date('Y-m', strtotime($time->created))) {
        // get the issue number
        $number = $issueNumbers[$time->issue_id];
        // if we've not already got a total for this issue
        // then create the entry, looking up issue title
        if (!isset($totals[$number])) {
            $issue = $repo->getIssue($number);
            $totals[$number] = [
                'id' => $number,
                'name' => $issue->title,
                'time' => 0,
            ];
        }
        // add the time to the total
        $totals[$number]['time'] += $time->time;
    }
}

// helper to format seconds into H:M:S format
function format_time($t)
{
    return sprintf('%02d:%02d:%02d', floor($t / 3600), ($t / 60) % 60, $t % 60);
}

// save the report
$f = fopen('report.csv', 'w');
// ouput headers
fputcsv($f, ['ID', 'Description', 'Hours', 'H:M:S']);
// output data
foreach ($totals as $id => $data) {
    $data['hours'] = $data['time']/60/60; //decimal hours
    $data['hms'] = format_time($data['time']); //H:M:S
    unset($data['time']); //dont want seconds in the report
    fputcsv($f, $data);
}
fclose($f);
echo "Saved as report.csv\n";
